/* "main.c" */
/* --------------------------------------------------------------------- */
/* 				EL FAITIN				 */
/* Joselo Balderas							 */
/* Artemio Urbina							 */
/* 						 	    octubre 2000 */
/* --------------------------------------------------------------------- */

#include    <sgl.h>
#include    <sega_sys.h>
#include    "main.h"


extern PDATA		pdata_cube1[];
extern TEXTURE 		Textures[];
extern PICTURE 		Sprites[];
extern SPR_ATTR 	Sprite_Atr[];
extern FIXED 		Posicion[][];
extern FIXED 		Angulo[];

#define     SystemWork      0x060ffc00              /* System Variable Address */
#define     SystemSize      (0x06100000-0x060ffc00) /* System Variable Size */

extern Uint32   _bstart, _bend;

static void Pal2CRAM(Uint16 *PalData, void *ColAddr, Uint32 PalSize)
{
	Uint16 i;
	Uint16 *VRAM;

	VRAM = (Uint16 *)ColAddr;

	for(i = 0; i<PalSize; i++)
		*(VRAM++) = *(PalData++);
}


void LoadSprites(PICTURE *pcptr , Uint32 NbPicture)
{
	TEXTURE *txptr;
 
	for(; NbPicture-- > 0; pcptr++){
		txptr = Textures + pcptr->texno;
		slDMACopy((void *)pcptr->pcsrc,
			(void *)(SpriteVRAM + ((txptr->CGadr) << 3)),
			(Uint32)((txptr->Hsize * txptr->Vsize * 4) >> (pcptr->cmode)));
	}
}
  
static void DisplaySprite(int cual)
{
	slDispSprite((FIXED*)Posicion[cual],
			(SPR_ATTR*)&Sprite_Atr[cual].texno,
			DEGtoANG(Angulo[cual]));
}

void InitializeSys()
{
    Uint8   *dst;
    Uint16  i;

    /* 1.Zero Set .bss Section */
    for(dst = (Uint8 *)&_bstart; dst < (Uint8 *)&_bend; dst++)
		*dst = 0;

    /* 2.SGL System Variable Clear */
    for(dst = (Uint8 *)SystemWork, i = 0; i < SystemSize; i++)
		*dst = 0;
}

void main(void)
{
    Uint16 PadData;
    static ANGLE ang[XYZ];
    static FIXED pos[XYZ];
    static FIXED scale[XYZ];
    static FIXED light[XYZ];

    InitializeSys();
    slInitSystem(TV_320x224, Textures, 1);

    slColRAMMode(CRM16_2048);
    LoadSprites(Sprites, NoSprites);
    slBack1ColSet((void *)BACK_CRAM,CD_Black);
       
    ang[X] = ang[Z] = DEGtoANG(0.0);
    ang[Y] = DEGtoANG(20.0);
    pos[X] = pos[Y] = toFIXED(0.0);
    pos[Z] = toFIXED(170.0);
    
    scale[X] = scale[Y] = scale[Z] = toFIXED(15.0);
    
    Pal2CRAM((Uint16 *)Sprite1Pal,(void *)VDP2_COLRAM,Sprite1PalSize);
    
    light[X] = toFIXED(-1.0);
    light[Y] = toFIXED( 0.0);
    light[Z] = toFIXED( 0.0);
    slLight(light);

    while(1)
    {
	slPushMatrix();
	{
		slTranslate(pos[X],pos[Y],pos[Z]);
		slRotX(ang[X]);
		slRotY(ang[Y]);
		slRotZ(ang[Z]);
		slScale(scale[X], scale[Y], scale[Z]);
		
		if(Per_Connect1)
    		{
    			PadData = Smpc_Peripheral[0].data;
    			if(PER_DGT_TX & PadData)
    			{
    				scale[X] += toFIXED(1.0);
    				scale[Y] += toFIXED(1.0);
    				scale[Z] += toFIXED(1.0);
    			}
    			if(PER_DGT_TA & PadData)
    			{
    				scale[X] -= toFIXED(1.0);
    				scale[Y] -= toFIXED(1.0);
    				scale[Z] -= toFIXED(1.0);
    			}
    			if(PER_DGT_TY & PadData)	
    				ang[X] += DEGtoANG(1.0);
    			if(PER_DGT_TB & PadData)	
    				ang[X] -= DEGtoANG(1.0);
    			if(PER_DGT_TZ & PadData)	
    				ang[Z] += DEGtoANG(1.0);
    			if(PER_DGT_TC & PadData)	
    				ang[Z] -= DEGtoANG(1.0);
    			if(PER_DGT_TR & PadData)
    				ang[Y] += DEGtoANG(1.0);
			if(PER_DGT_TL & PadData)
    				ang[Y] -= DEGtoANG(1.0);
    				
    				
    			if(PER_DGT_KU & PadData)
    				Posicion[0][1] += toFIXED(1.0);
    			if(PER_DGT_KD & PadData)	
    				Posicion[0][1] -= toFIXED(1.0);
    			if(PER_DGT_KR & PadData)	
    				Posicion[0][0] -= toFIXED(1.0);
    			if(PER_DGT_KL & PadData)	
    				Posicion[0][0] += toFIXED(1.0);
    				
    			if(!(PER_DGT_ST & PadData))
    			{
    				Posicion[0][0] = toFIXED(-64.0);
    				Posicion[0][1] = toFIXED(-16.0);
    				Posicion[1][0] = toFIXED(64.0);
    				Posicion[1][1] = toFIXED(-16.0);
    				ang[X] = ang[Z] = DEGtoANG(0.0);
    				ang[Y] = DEGtoANG(20.0);
    				pos[X] = pos[Y] = toFIXED(0.0);
    				pos[Z] = toFIXED(170.0);
    				Angulo[0] = Angulo[1] = 0;
    			}
    				
    		}
    		else
    		{
    			ang[Y] += DEGtoANG(1.0);
    			Angulo[0] += 1;
    			Angulo[1] -= 1;
    		}
    		
    		if(Per_Connect2)
    		{
    			PadData = Smpc_Peripheral[15].data;
    			if(PER_DGT_TX & PadData)
    				pos[Z] += toFIXED(10.0);
    			if(PER_DGT_TA & PadData)
    				pos[Z] -= toFIXED(10.0);
    			if(PER_DGT_TY & PadData)	
    				ang[X] += DEGtoANG(10.0);
    			if(PER_DGT_TB & PadData)	
    				ang[X] -= DEGtoANG(10.0);
    			if(PER_DGT_TZ & PadData)	
    				Angulo[0] += 1;
    			if(PER_DGT_TC & PadData)	
    				Angulo[0] -= 1;
    			if(PER_DGT_TR & PadData)
    				Angulo[1] += 1;
			if(PER_DGT_TL & PadData)
    				Angulo[1] -= 1;
    				
    				
    			if(PER_DGT_KU & PadData)
    				Posicion[1][1] += toFIXED(1.0);
    			if(PER_DGT_KD & PadData)	
    				Posicion[1][1] -= toFIXED(1.0);
    			if(PER_DGT_KR & PadData)	
    				Posicion[1][0] -= toFIXED(1.0);
    			if(PER_DGT_KL & PadData)	
    				Posicion[1][0] += toFIXED(1.0);    				    				
    				
    		}
  
		slPutPolygon(pdata_cube1);
	}
	slPopMatrix();
    
	DisplaySprite(0);
	DisplaySprite(1);
	slSynch();
    } 
}
